/* * This file is part of contactui *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef AVATARWIDGET_H
#define AVATARWIDGET_H

#include <QUrl>
#include <QFile>
#include <qtcontacts.h>

#include <MImageWidget>

#include <avatarwidgetmodel.h>

class MLabel;

/**
 * \class AvatarWidget
 * \brief This is the widget that displays the avatar image and presence for a contact. Optionally,
 * It displays contacts name too.
 * Online presence is displayed by overlay graphics in corner of avatar, in colors depicting presence.
 * If contact object is null or has no avatar defined then the default avatar image is shown.
 *
 * There are 3 ways to use Avatar widget:
 *
 * 1) To supply data from outside through setPresenceStatus, setAvatar, setContact.
 * In this case avatar model is not making request to libqtcontacts to fetch data. This is useful
 * when presenting multiple avatars in List - as ListModel already holds and updates needed data.
 * When update happens, setContact or setAvatar is called again and Avatar UI gets refreshed
 *
 * 2) to supply contactId (either in constructor or via \sa setContactId()).
 * In this case avatar model makes asynchronous request to fetch data from libqtcontacts and is responsible
 * for updating itself when content changes in contacts storage.
 *
 * 3) If multiple avatars for the same contact need to be displayed, then it is suggested to
 * use \sa AvatarWidget(AvatarWidgetModel* model, MWidget* parent = 0) or \sa setModel().
 * That way avatars share the same instance of model, and, if this is used in combination with 2, only one
 * request to libqtcontacts is made, and only one request for updating data on data change signal from
 * libqtcontacts.
 *
 * AvatarWidget supports Direct UI MVC pattern - It is important
 * to define default view for avatar in <target>.conf file:
 *  [AvatarWidget]
 *  default = AvatarWidgetView
 *
 * Otherwise, avatar widget will not be displayed.
 *
 *
 *
 **/
class Q_DECL_EXPORT AvatarWidget : public MImageWidget
{
    Q_OBJECT
    M_CONTROLLER(AvatarWidget)
public:
    /**
     * \brief Creates the AvatarWidget with empty avatar photo.
     * This one is more natural to be used in edit mode - where avatar needs to
     * show image that is preset through \sa AvatarWidget::setAvatar
     *
     * Additionally if you are already querying QContactManager, it would be more efficient
     * to use this constructor and then use setAvatar and setPresenceStatus. Setting
     * the contactId would setup redundant queries.
     *
     * \param parent parent object
     **/
    explicit AvatarWidget(MWidget* parent = 0);

    /*!
     * Enables model reusing - sharing between multiple widget instances.
     * e.g. AvatarWidget newOne(oldOne.model(), parent);
     * This way newOne and oldOne are sharing the model and only one
     * request to data is issued
     */
    explicit AvatarWidget(AvatarWidgetModel* model, MWidget* parent = 0);

    /**
     * \brief Creates the AvatarWidget for the contact. Widget refreshes itself as
     * contact's avatars and presence statuses change so no need to call \sa setPresenceStatus
     * and \sa setAvatar from outside.
     * TODO clarify MVP. contactID is something pointing to QAIM element
     *
     * \param parent parent object
     **/
    explicit AvatarWidget(const QContactLocalId& contactId, MWidget* parent = 0);

    //!\instance is about to end
    virtual ~AvatarWidget();

public slots:
    /*!
     * Shows contact, updates itself when contact's model changes
     * \sa AvatarWidget(const QString& contactId, MWidget* parent)
     */
    void setContactId(const QContactLocalId& contactId);

    /*!
     * Shows contact using an existing QContact
     */
    void setContact(const QContact& contact);

    /**
     * Set the current presence state.
     * It will be represented by an icon overlayed on the avatar.
     */
    void setPresenceState(QContactPresence::PresenceState presenceState);

    /**
     * Returns the current presence status represented by this widget
     */
    QContactPresence::PresenceState presenceState() const;

    /**
     * Returns the current status message (eg "Out Walking the Dog")
     * This method has no relevance to the widget, and is provided purely
     * as a convenience. It only works if a ConctactId has been provided.
     **/
    QString statusMessage() const;

    /**
     * Sets the current avatar image represented by this widget.
     * If the avatar is empty, a default avatar image is set instead.
     */
    void setAvatar(const QString& avatar);

    /**
     * Sets the current avatar image represented by this widget.
     \param url Local url to avatar image file
     */
    void setAvatar(const QUrl& url);

    /*!
     * Sets the current avatar \a pixmap represented by this widget.
     */
    void setAvatarPixmap(const QPixmap &pixmap);

    // not sure if this will remain in API  - or moved to
    void setNameLabelDisplayed(bool visible);

    void setContactNameInitials(const QString &nameInitials);

    void setContactDisplayLabel(const QString &displayLabel);

    /*!
     * Sets the current waiting for approval state i.e. avatar of contact that has been requested as a friend, but the remote end
     * has not yet approved this request.
     * \remarks This don't have to be used if you use a\ setContact or \a setContactId
     */
    void setWaitingForApproval(bool isWaiting);

    /*!
     * \returns the current state of waiting for approval. \a setWaitingForApproval.
     */
    bool isWaitingForApproval() const;

    /*!
     * Sets whether the widget should consume mouse event. Default is true.
     */
    void setConsumeMouseEvent(bool consume);

    /*!
     * \returns the current state of consume mouse event. \a setConsumeMouseEvent.
     */
    bool consumeMouseEvent();

    /*!
     * Sets whether the widget should show down state effect. Default is false.
     * \remarks Consume mouse event needs to be set TRUE to get effect visible.
     */
    void setShowDownStateEffect(bool show);

    /*!
     * \returns the current state of showing down state effect. \a setShowDownStateEffect.
     */
    bool showDownStateEffect();

public:
    /**
     * returns the ID of the contact the widget is presenting
     */
    QContactLocalId contactId();

    /*!
     * Emits clicked() signal.
     */
    void click();

signals:
    /**
     * Emitted when the avatar image is clicked.
     */
    void clicked();

protected:
    /**
     * Reimplement QGraphicsItem::mousePressEvent() to be able to emit clicked()
     * signal to our own components.
     * \reimp
     */
    void mousePressEvent( QGraphicsSceneMouseEvent * event );

    QSizeF sizeHint(Qt::SizeHint which, const QSizeF & constraint = QSizeF()) const;

private:
    void initializeWidget(const QString& avatar, QContactPresence::PresenceState presenceState);
    void initSearchPaths();
    void setTemporaryContact();

private slots:


private:
    class Private;
    Private *p;
};

#endif // AVATARWIDGET_H
