/* * This file is part of libcontacts *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */

#ifndef CONTACTACCOUNT_H_
#define CONTACTACCOUNT_H_

#include <QObject>

#include <Accounts/Account>

#include "contactaccountservice.h"
#include "contactaccountpath.h"

typedef QPair<Accounts::AccountId, QString> ServiceId;

/*!
 * \class ContactAccount
 * \brief Class contening cached information about account
 */
class ContactAccount : public QObject
{
    Q_OBJECT

public:
    enum Flags {
        None = 0,           // No flag
        Enabled = 1,        // Account is enabled
        OnlyMatching = 2,   // Only contacts matching existing ones are imported
        HasCall = 4         // Account is supporting call
    };

    ContactAccount(ServiceId id);

    virtual ~ContactAccount();

    /*!
     * Service Id to refer to specific service from an account
     */
    const ServiceId& serviceId() const { return mServiceId; }

    /*!
     * Account Id used by Accounts framework.
     */
    Accounts::AccountId accountId() const { return mServiceId.first; }

    /*!
     * Return whether the account matches given flags
     */
    bool isMatching(int flags);

    /*!
     * Return the first non-null account path (telepahy path or social one)
     */
    const QString& accountPath() const { return mTelepathyPath.isEmpty() ? mSocialPath : mTelepathyPath; }
    
    /*!
     * Telepathy object path for IM accounts.
     */
    const QString& telepathyPath() const { return mTelepathyPath; }
    void setTelepathyPath(const QString& path) { mTelepathyPath = path; }

    /*!
     * Social account path of form PROVIDER:ACCOUNT_NAME.
     */
    const QString& socialPath() const { return mSocialPath; }
    void setSocialPath(const QString& path) { mSocialPath = path; }

    /*!
     * Account name (e.g. gtalk: 'user@gmail.com', skype: 'username').
     */
    const QString& accountName() const { return mAccountName; }
    void setAccountName(const QString& name) { mAccountName = name; }

    /*!
     * Service information (display name, icon)
     */
    ContactAccountService* service() const { return mService; }
    void setService(ContactAccountService *service) { mService = service; }

    /*!
     * Provider information (display name, icon)
     */
    ContactAccountProvider* provider() const { return mProvider; }
    void setProvider(ContactAccountProvider *provider) { mProvider = provider; }

    /*!
     * Account protocol (e.g. 'jabber', 'skype', 'sip').
     */
    const QString& protocol() const { return mProtocol; }
    void setProtocol(const QString& protocol) { mProtocol = protocol; }

    /*!
     * Service icon.
     */
    const QString serviceIcon() const { return mService ? mService->icon() : QString(); }

    /*!
     * Service display name (e.g. 'Google Talk', 'Facebook Chat', 'Skype').
     */
    const QString serviceName() const { return mService ? mService->displayName() : QString(); }

    /*!
     * Provider icon. Used when referring to that account in UI.
     */
    const QString providerIcon() const { return mProvider ? mProvider->icon() : QString(); }

    /*!
     * Provider localized name (e.g. 'Google', 'Facebook', 'Skype').
     */
    const QString providerName() const { return mProvider ? mProvider->displayName() : QString(); }

    /*!
     * Whether the account is currently enabled.
     */
    bool isEnabled() const { return mEnabled; }
    void setEnabled(bool enabled) { mEnabled = enabled; }

    /*!
     * Whether only contacts matching existing ones are imported from
     * that account.
     */
    bool isImportOnlyMatchingContacts() const { return mOnlyMatching; }
    void setImportOnlyMatchingContacts(bool onlyMatching);

    /*!
     * Whether the account supports calls.
     */
    bool hasCall() const { return mHasCall; }
    void setHasCall(bool supported) { mHasCall = supported; }

    bool operator<(const ContactAccount &a2) const
    {
        return this->serviceName().toLower() < a2.serviceName().toLower();
    }

    bool operator<(const ContactAccount *a2) const
    {
        return this->serviceName().toLower() < a2->serviceName().toLower();
    }

signals:
    void importOnlyMatchingContactsChanged(const QString &accountPath, ContactAccountPath::PathType type, bool onlyMatching);

private:
    ServiceId mServiceId;
    ContactAccountService *mService;
    ContactAccountProvider *mProvider;
    QString mTelepathyPath;
    QString mSocialPath;
    QString mProtocol;
    QString mAccountName;
    bool mEnabled;
    bool mOnlyMatching;
    bool mHasCall;
};

#endif /* CONTACTACCOUNT_H_ */
