/* * This file is part of libcontacts *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright
 * controlled by Nokia Corporation. All rights are reserved. Copying,
 * including reproducing, storing, adapting or translating, any or all of
 * this material requires the prior written consent of Nokia Corporation.
 * This material also contains confidential information which may not be
 * disclosed to others without the prior written consent of Nokia.
 */

#ifndef CONTACTACCOUNTMANAGER_H_
#define CONTACTACCOUNTMANAGER_H_

#include <QMap>
#include <QObject>
#include <QPair>

#include <QContactOnlineAccount>
#include <QContactPresence>

#include <Accounts/Account>

#include "contactaccount.h"

QTM_USE_NAMESPACE

class ContactAccountService;
class ContactAccountProvider;

namespace Accounts {
    class Manager;
}

/*!
 * \class ContactAccountManager
 * \brief This controls any account and returns information about them
 *
 * This class make sure we don't have to retrieve information from Accounts
 * framework everytime we need some info about a service. As the number of
 * configured accounts is relatively low, we cache information about these
 * in this manager and monitor any changes (adding, removing, enabling ...)
 *
 * \see ContactAccountProvider, ContactAccountService and ContactAccount
 * to know exactly what information is cached.
 */
class Q_DECL_EXPORT ContactAccountManager : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(ContactAccountManager)
    Q_ENUMS(DetailOrder)

private:
    ContactAccountManager();

public:
    virtual ~ContactAccountManager();
    static ContactAccountManager* instance();

    enum DetailOrder {
        SingleServiceName = 0,
        SingleUserName,
        ServiceNameAndUserName,
        UserNameAndStatus
    };

    /*!
     * Return information about all cached accounts
     */
    QList<ContactAccount*> accounts() const;

    /*!
     * Return information about all accounts using given protocol and matching flags.
     * \param protocol Protocol name such as 'sip', 'skype', etc.
     * \param flags Values of ContactAccount::Flags enum
     */
    QList<ContactAccount*> accountsFor(const QString &protocol) const;
    QList<ContactAccount*> accountsFor(const QString &protocol, int flags) const;

    /*!
     * Return information about all accounts using given service and matching flags.
     * \param service Service name such as 'facebook', 'google-talk', etc.
     * \param flags Values of ContactAccount::Flags enum
     */
    QList<ContactAccount*> accountsForService(const QString &service, int flags) const;

    /*!
     * Return information about all accounts matching given flags.
     * \param flags Values of ContactAccount::Flags enum
     */
    QList<ContactAccount*> accountsMatching(int flags) const;

    /*!
     * Return information about an account (account name, provider, icon...).
     * \see ContactAccount to know what information are cached.
     * \param accountPath Path of the account whose info are looked up
     */
    ContactAccount* accountInformation(const QContactOnlineAccount &account) const;

    /*!
     * Return information about an account (account name, provider, icon...).
     * \see ContactAccount to know what information are cached.
     * \param accountPath Path of the account whose info are looked up
     */
    ContactAccount* accountInformation(const QString &accountPath) const;

    /*!
     * Return information about a service (display name, icon...).
     * \see ContactAccountService to know what information are cached.
     * \param name Service name such as 'google-talk', 'msn-nm', etc.
     */
    ContactAccountService* serviceInformation(const QString &name) const;

    /*!
     * Return information about a provider (display name, icon...).
     * \see ContactAccountProvider to know what information are cached.
     * \param name Provider name such as 'google', 'facebook', etc.
     */
    ContactAccountProvider* providerInformation(const QString &name) const;

    /*!
     * Return whether the given account is read-only. It's impossible to
     * add contacts from these accounts. As of now, it's hardcoded
     * to "facebook" and "twitter" as the user needs to use specific
     * application to add \a friends for these providers.
     */
    bool isReadOnlyAccount(const QContactOnlineAccount &account) const;

    /*!
     * Return whether the given social account is removable.
     * It's only possible to remove contacts from these accounts.
     * As of now, it's hardcoded to "twitter"
     */
    bool isRemovableSocialAccount(const QContactOnlineAccount &account) const;

    /*!
     * Return whether the given account is an \a online one. This is used
     * to filter accounts so that \a cellular accounts (tp-ring, mms) don't
     * appear in the UI.
     */
    bool isOnlineAccount(const QContactOnlineAccount &account) const;

    /*!
     * Return whether the given account is roster-less. (i.e. there is
     * no contact list for that protocol).
     */
    bool isRosterlessAccount(const QContactOnlineAccount &account) const;

    /*!
     * Return whether the given account supports making VOIP calls.
     * This is used to override the telepathy capabilities in case a contact is offline.
     * At the moment, this is hardcoded to gtalk and skype.
     */
    bool supportsVoip(const QContactOnlineAccount &account) const;

    /*!
     * Return whether the \a "import-only-matching-contacts" setting has
     * been set to true for that account.
     * /return True if only contacts matching already existing ones should
     * be imported for that account. False otherwise (import all contacts).
     */
    bool isImportOnlyMatchingContacts(const QContactOnlineAccount &account) const;
    bool isImportOnlyMatchingContacts(const QString &accountPath) const;

    /*!
     * For some services QContactOnlineAccount::accountUri() is ugly/useless, so we tweak it to get usable string.
     * Return account username, which is IM address for IM contacts, and nickname or screen name for social contacts.
     * \param account Online account of the contact.
     * \param presence Linked presence of the \p account. IM nickname is included in the presence.
     */
    QString onlineAccountDisplayLabel(const QContactOnlineAccount &account, const QContactPresence &presence);

    /*!
     * Return custom detail order for service in case of single item of that service in UI
     */
    ContactAccountManager::DetailOrder detailOrderForSingleService(const QContactOnlineAccount &account);

    /*!
     * Return custom detail order for service in case of multiple items of that service in UI
     */
    ContactAccountManager::DetailOrder detailOrderForMultipleService(const QContactOnlineAccount &account);

    /*!
     * Load settings from cache and emit signal for any setting that has changed
     * since last execution (current value different than cached one).
     */
    bool loadCache();

    /*!
     * Store current settings in cache (only ImportOnlyMatching for now).
     */
    bool storeCache();

signals:
    /*!
     * Emitted if any account is added, removed or updated.
     */
    void accountsChanged();

    /*!
     * Emitted if account is added and enabled
     */
    void accountEnabled(const QString &serviceName);

    /*!
     * Emitted when \a "import-only-matching-contacts" setting has changed
     * for an account.
     */
    void importOnlyMatchingContactsChanged(const QString &accountPath, ContactAccountPath::PathType type, bool onlyMatching);

private:
    QString accountPath(const QContactOnlineAccount &account) const;

    ContactAccount* createAccount(const ServiceId &serviceId, Accounts::Account *account);
    ContactAccountService* retrieveService(const QString &name);
    ContactAccountService* createService(const QString &name);
    ContactAccountProvider* retrieveProvider(const QString &name);
    ContactAccountProvider* createProvider(const QString &name);

    bool hasAccount(const QString &accountPath) const;
    void updateAccountsList();
    void timerEvent(QTimerEvent *event);
    void addTranslationCatalog(const QString &catalog);
    bool isFacebookProvider(const QString &providerName) const;
    bool isTwitterProvider(const QString &providerName) const;
    bool isGoogleProvider(const QString &providerName) const;
    bool isSkypeProvider(const QString &providerName) const;

    Accounts::Provider *accountsProvider(const QString &name);

private slots:
    void onAccountEnabledChanged(const QString &serviceName);
    void onServiceUpdated();
    void updateAccount(Accounts::Account *account);
    void updateAccount(Accounts::AccountId accountId);
    void removeAccount(Accounts::AccountId accountId);

private:
    Accounts::Manager *mAccountManager;
    QMap<ServiceId, ContactAccount *> mAccountMap;
    QMap<QString, ContactAccount *> mAccountPathMap;
    QMap<QString, ContactAccountService *> mServiceMap;
    QMap<QString, ContactAccountProvider *> mProviderMap;
    QMap<ServiceId, Accounts::Watch *> mServiceWatches;

    QList<Accounts::AccountId> mPendingAccounts;
    int mPendingTimer;

friend class ContactsService;
};

#endif /* CONTACTACCOUNTMANAGER_H_ */
