/* * This file is part of contactui *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTS_ACTIONS_CONTACTACTION_H
#define CONTACTS_ACTIONS_CONTACTACTION_H

// Qt includes
#include <QContact>

#include <MAction>

// People & Contacts includes
#include "contactsguiactionresult.h"

QTM_USE_NAMESPACE

class IContactHolder;;

/**
 * \class ContactAction
 * \brief This is a base class for actions that operate on a particular QContact.
 */
class ContactAction : public MAction
{
    Q_OBJECT
public:

    explicit ContactAction(const QString &iconID, const QString &text, QObject *parent=0);

    virtual ~ContactAction();
    
    /*!
     * Returns current action contact
     */
    const QContact& contact() const;

    /*!
     * Sets the contact to be changed.
     */
    virtual void setContact(const QContact &contact);

    /*!
     * Sets contact and merged contacts.
     */
    virtual void setContact(const QContact &contact, const QList<QContact> &mergedContacts);

    /*!
     * Sets the contact holder to be changed.
     */
    virtual void setContactHolder(IContactHolder *contactHolder);

    /*!
     * Update contact fetching needed before the operation starts.
     */
    void setFetchContact(const bool fetchContact);

Q_SIGNALS:
   /** Derived classes should emit this when the operation is really finished.
    * \param success - whether this operation is successful or failed
    * @see doOperation().
    */
   void operationDone(bool success);

protected:
    //Either one of these is used, depending on the constructor used, but never both:
    QContact mContact;
    IContactHolder *mContactHolder;
    ContactsGuiActionResult::SharedPointer mGuiActionResult;
    QList<QContact> mMergedContacts;

private Q_SLOTS:
    void onContactChanged();
    void onTriggered();
    void onRequestProgress();

private:
    /*!
     * Implement this to specify the minimum set of QContact details
     * that will be retrieved during fetching.
     * Method is invoked on trigger() - if setFetchContact(true), first
     * defined details are fetched, then doOperation is invoked
     */
    virtual QStringList detailDefinitionsHint() const = 0;

    /** Implement this to perform the operation on the contact 
     * after the action has been triggered.
     * If necessary, the contact will be fetched first, before this 
     * is called.
     * This should emit the operationDone signal when the operation is fully completed.
     */
    virtual void doOperation() = 0;

    /// Override this to update the action text when the contact changes.
    virtual void updateText();

    bool mFetchingNeeded;
};

#endif // CONTACTS_ACTIONS_CONTACTACTION_H
