/* * This file is part of contacts/contactui*
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTDETAILSELECTOR_H_
#define CONTACTDETAILSELECTOR_H_

#include <QModelIndex>
#include <QList>
#include <QContact>
#include <QContactDetail>

#include <MDialog>

class ContactDetailSelectorPrivate;
class MDialog;

QTM_USE_NAMESPACE

/*!
 * \class ContactDetailSelector
 * \brief
 * ContactDetailSelector is a dialog for selecting one or several details of a contact.
 *
 * After users select a contact from contacts picker, the contact might contain
 * more than one contact details such as phone number or email address,
 * users might want to select one or several contact details for communication.
 * Likewise, user may also want to pick several contacts and details for communication.
 *
 * Example usage:
 * \code
 *
 * ContactDetailSelector *selector = new ContactDetailSelector(contact, details, highlightedDetails, ContactDetailSelector::SingleSelection);
 * connect(selector, SIGNAL(detailsSelected(QList<QContactDetail>)),
 *         this, SLOT(onDetailsSelected(QList<QContactDetail>)));
 * selector->appear(MApplication::instance()->activeWindow(), MSceneWindow::DestroyWhenDone);
 *
 * \endcode
 */
class Q_DECL_EXPORT ContactDetailSelector : public MDialog
{
    Q_OBJECT

public:
    /**
     * \enum SelectionMode
     * Whether selecting one or several contact details
     * \value SelectionMode Select one contact details
     * \value MultiSelection Select several contact details
     */
    enum SelectionMode {
        SingleSelection = 0,
        MultiSelection
    };

    /**
     * \brief Constructor.
     *
     * \param contact - details owner
     * \param details - contact details to be shown
     * \param selectionMode - selecting single or several details
     */
    explicit ContactDetailSelector(const QContact &contact, const QList<QContactDetail> &details,
                                   ContactDetailSelector::SelectionMode selectionMode = ContactDetailSelector::SingleSelection);

    /**
     * \brief Constructor.
     *
     * \param contact - details owner
     * \param details - contact details to be shown
     * \param highlightedDetails - contact details to be highlighted
     * \param selectionMode - selecting single or several details
     */
    explicit ContactDetailSelector(const QContact &contact, const QList<QContactDetail> &details, const QList<QContactDetail> &highlightedDetails,
                                   ContactDetailSelector::SelectionMode selectionMode = ContactDetailSelector::SingleSelection);

    ~ContactDetailSelector();

signals:
    /**
     * Emit this signal after one or more contact details are selected
     */
    void detailsSelected(const QList<QContactDetail> &);

private slots:
    // slot for single selection
    void onListItemClicked(const QModelIndex &index);

    // slot for multi selection
    void onDoneButtonClicked();

private:
    void createContent(const QContact &contact, const QList<QContactDetail> &details);

    QList<QContactDetail> filterInvalidDetails(const QList<QContactDetail> &details);

    ContactDetailSelectorPrivate * const d;
};

#endif /* CONTACTDETAILSELECTOR_H_ */
