/* * This file is part of contacts/contactui *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTLISTENER_H
#define CONTACTLISTENER_H

// Qt includes
#include <QObject>
#include <QStringList>
#include <QQueue>

// Qt Contact includes
#include <qcontact.h>
#include <qcontactid.h>
#include <qcontactabstractrequest.h>
#include <qcontactfetchrequest.h>
#include <qcontactmanager.h>

// People & Contacts includes
#include "icontactholder.h"
#include <contactmodel.h>

QTM_USE_NAMESPACE

/**
 * \class ContactListener
 * @todo ContactsListener is caching contact data change listener for a given contact ID.
 */
class ContactListener : public QObject, public IContactHolder
{
    Q_OBJECT

public:
    /**
     * \brief Constructor.
     * \param parent QObject parent of this class.
     */
    explicit ContactListener(QObject *parent = 0);

    /**
     * \brief Desctructor.
     */
    virtual ~ContactListener();

public: // IContactHolder API
    /**
     * \brief Returns reference to contact this model represents.
     * \return Contact.
     */
    virtual const QContact & contact() const;

    /**
     * \brief Returns list of merged contacts
     */
    virtual QList<QContact> mergedContacts() const;

public:
    /**
     * \brief Sets contact ID for the model.
     * \param id Contact ID.
     */
    void setContactId(const QContactId &localId);

    void setContactId(const QContactLocalId &localId, const QString &managerUri);

    /**
     * \brief Returns QContactId for the contact this model listens
     */
    QContactId contactId() const;

    /**
     * \brief Enable/disable signals when the contact is changed/updated.
     * For some operations (merging/unmerging), the contact card content
     * has to stay the same until the fade out animation is finished. To
     * achieve that, signaling is disabled temporarily.
     */
    void enableSignals();
    void disableSignals();

    /**
     * \brief Add list of @p definitionNames to follow by this model.
     * \param definitionNames Possibly new definition names to follow.
     */
    void addDefinitionNames(const QStringList &definitionNames);

    /**
     * \brief Add list of @p otherDefinitionNames to follow by this model.
     * \param definitionNames Possibly new definition names to follow.
     */
    void addOtherDefinitionNames(const QStringList &definitionNames);

    /**
     * \brief Returns a list of followed definition names.
     * \return Followed definition names.
     */
    const QStringList & definitionNames() const;

    /**
     * \brief Returns a list of followed additionnaly definition names.
     * \return Followed definition names.
     */
    const QStringList & otherDefinitionNames() const;

signals:
    /**
     * \brief Signal to inform that listener has received (initial) contact.
     * \param empty Flag to indicate if received contact is empty (no contact with that ID in tracker).
     */
    void contactReceived(bool empty);

    /**
     * \brief Signal to inform contact monitored by listener has changed.
     * \param id Id of new listened contact
     */
    void contactChanged(const QContactId &id);

    /**
     * \brief Signal to inform contact has been updated.
     * \param definitionNames List of definitionnames which details are changed
     */
    void contactUpdated(const QStringList &definitionNames);

    /**
     * \brief Signal to inform that contact we were interested in was removed.
     * \param id Removed contact id.
     */
    void contactRemoved(QContactLocalId id);

protected slots:
    void onContactsUpdated (const QList<QContact> &contacts, const QStringList &updatedDetails);
    void onContactsAdded (const QList<QContactId> &contactIds);
    void onContactsRemoved (const QList<QContactId> &contactIds);
    void onContactsNotFound(const QString& managerUri, const QList<QContactLocalId> &ids);
    void onContactsMerged(const QList<QContactRelationship> &relationships);
    void onContactsUnmerged(const QList<QContactRelationship> &relationships);

protected:
    void requestContactInfo(bool signal = true);
    void setContactType(const QString &type);
    void updateContact(bool signal = true);

protected:
    //! Current contact
    QContact mContact;

    //! Current master contact (same as mContact if there is no merged contacts)
    QContact mMasterContact;

    //! Current merged contacts
    QMap<QContactId, QContact> mMergedContacts;

    //! ID of current contact
    QContactId mId;

    //! Type of current contact
    QString mType;

    //! Definition restcrictions given to QContactFetchRequest
    QStringList mDefinitionNames;

    //! Definition restcrictions given to connected detail plugins
    QStringList mOtherDefinitionNames;

private:
    ContactModel *mContactModel;
    bool mSignalEnabled;
    bool mPendingChange;
    bool mPendingUpdate;
};

#endif // CONTACTLISTENER_H
