#ifndef CONTACTLISTTEMDATA_H
#define CONTACTLISTTEMDATA_H

#include <QPixmap>
#include <QString>
#include <QObject>
#include <QMap>
#include <QSet>
#include <QModelIndex>
#include <QUuid>

#include <qtcontactsglobal.h>
#include <QContact>
#include <QContactOnlineAccount>
#include <QContactPresence>

QTM_USE_NAMESPACE


/** The ContactsTreeModel provides instances of this ContactListItemData class
 * from data(), in a QVariant.
 *
 * @todo Replace this with direct use of QContact if we never need anything extra?
 * //TODO_Performance: Using QContact directly might prevent some copying.
 */
class ContactListItemData
{
public:
    ContactListItemData();
    void setContact(const QContact &contact, const QList<QContact> &mergedContacts = QList<QContact>());
    void addMergedContact(const QContact &contact);
    void removeMergedContact(const QContactId &contactId);

    QContactId contactId() const { return mContact.id(); }
    QContactLocalId localId() const { return mContact.localId(); }
    QContact contact() const { return mContact; }
    QList<QContact> mergedContacts() const { return mMergedContacts.values(); }

    const QString &displayLabel() const;
    const QString &nameInitials() const;
    const QString &avatarFileName(const QString &accountPath = QString()) const;
    const QString &status() const;
    QContactPresence::PresenceState presence() const;
    bool isFavourite() const;
    bool isPendingApproval() const;
    bool isGroup() const;
    bool search(const QRegExp &rx);

    /*! This is used to uniquely identify a ContactListItemData instance to provide
     * quick lookup in ContactsGroupingModel, as it cannot easily identify source model rows, 
     * as they change.
     *
     * TODO: this can probably be removed if/when contactlistmodel is merged
     */
    const QUuid &modelHashId() const;

    /*! Determines whether or not the contact has a phone number.
     */
    bool hasPhoneNumber() const;

    /*! Determines whether or not the contact has a given \a accountPath.
     */
    bool hasAccount(const QString &accountPath) const;

    /*! Determines whether or not the contact supports a given \a action.
     */
    bool supportsAction(const QContactActionDescriptor &action) const;

    static void addAutoMergedContact(const QContactId &contactId);
    static bool isContactAutoMerged(const QContactId &contactId);
    static QList<QContactId> autoMergedContactIds();
    static void clearAutoMergedContact();

    /* Determines whether this data instance matches a given \a other data instance.
     * This is used e.g. to provide suggestions in the contact picker.
     */
    bool matches(const ContactListItemData *other) const;

    QString accountUri() const;

private:
    //Prevent copying:
    ContactListItemData(const ContactListItemData& src);
    const QContactOnlineAccount findAccount(const QString &accountPath) const;
    void addSearchStrings(const QContact &aContact);
    void clearTokens();

    // Keep track of all automatically merged contacts
    // remove them from the map once Contacts enter task switcher
    static QMap<QString, QSet<QContactLocalId> > mAutoMergedContacts;

    // mutable used for lazy loading - getters are const.
    mutable QString mDisplayLabel;
    mutable QString mNameInitials;
    mutable QMap<QString, QString> mAvatarFileNames;
    mutable QString mStatus;
    mutable QContactPresence::PresenceState mPresence;
    mutable bool mIsPendingImApproval;
    QStringList mSearchContent;

    QContact mContact;
    QMap<QContactId, QContact> mMergedContacts;
    bool mIsMasterSocial;
    QUuid mModelHashId;

    friend class TestContactsListModel;
};

// implementing deleteLater for list of ContactListItemData
class ContactListItemDataGarbageCan: public QObject {
Q_OBJECT
public:
    ContactListItemDataGarbageCan(): QObject(0) {
    }

    virtual ~ContactListItemDataGarbageCan() {
        qDeleteAll(trash.begin(), trash.end());
    }

    void toTrash(ContactListItemData *data) {
        trash << data;
    }
private:
    QList<ContactListItemData *> trash;
};

#endif // CONTACTLISTTEMDATA_H
