/* * This file is part of contacts/contactui*
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTLISTWIDGET_H
#define CONTACTLISTWIDGET_H

// Qt includes
#include <QDebug>
#include <QKeyEvent>

// Qt Mobility
#include <QContactId>

// M includes
#include <MKeyboardStateTracker>
#include <MLayout>
#include <MGridLayoutPolicy>
#include <MList>
#include <MOrientationChangeEvent>
#include <MPannableViewport>

#include "contactslistmodel.h"
#include "contactslistsettings.h"
#include "searchfieldwidget.h"

class MLabel;
class MStylableWidget;
class ContactsGroupingModel;
class ContactListItemBuilder;
class ContactsListModel;
class ContactSearchWidget;
class SearchFieldWidget;
class MProgressIndicator;

/*!
 * \class ContactListWidget
 * \brief
 * Widget presenting contacts as a list.
 * Used for both contact list and picker
 * Containing single MList, search widget and
 * linking ContactModel to m list for sorting and filtering
 */
class ContactListWidget : public MPannableViewport
{
    Q_OBJECT

public:
    /**
     * \brief The contact list page content item indexes.
     */
    enum ContactListWidgetElements {
        CONTACT_IMPORT_PROGRESS = 0,
        CONTACT_SEARCH,
        CONTACT_TOP_STRETCH,
        CONTACT_LIST,
        CONTACT_BOTTOM_STRETCH
    };

    /**
     * \brief Create the object.
     **/
    ContactListWidget(ContactsGroupingModel *modelSorted, MList::SelectionMode mode = MList::NoSelection);

    /**
     * \brief Create the object. To avoid binary break, overloading previous to supply \a parent.
     * Note: Supply parent to avoid stylesheet reloading (costs ~20 ms)
     **/
    ContactListWidget(ContactsGroupingModel *modelSorted, MList::SelectionMode mode, QGraphicsItem *parent);

    ContactListWidget(ContactsGroupingModel *modelSorted, MList::SelectionMode mode, QGraphicsItem *parent, bool showSearch);

    /**
     * \brief Destruct object.
     **/
    ~ContactListWidget();

    /**
     * \brief Create widget content
     */
    void createContent();

    /**
     * \brief Set empty state text
     */
    void setEmptyStateText(const QString &emptyStateText);

    /**
     * \brief Returns if there is currently a search filter
     */
    bool isFiltered() const;

    /**
     * \brief Set a string to filter the contact list
     *
     * \param filter - a string to filter the contact list
     */
    void setFilter(const QString& string);

    /**
     * \brief Clear the contact list filter
     */
    void clearFilter();

    /**
     * \brief Returns the current filter
     *
     * \returns - a string containing the current filter
     */
    QString filter();

    /**
     * \brief Returns the list's model
     */
    ContactsGroupingModel *model() const;

    /**
     * \brief Set groupBy mode (by name or by availability)
     */
    void setGroupBy(ContactsListSettings::GroupByType groupBy);

    /**
     * \brief Set whether the viewport is pannable
     */
    void setPannable(bool pannable);

    /**
     * \brief Scroll to the top of the viewport
     */
    void scrollToTop();

    /**
     * \brief Ensure a contact is centered as soon as it's added
     */
    void scrollToNewContact(const QContactId &contactId);

    /**
     * \brief Scroll the list to center the contact
     */
    void scrollToContact(const QContactId &contactId);

    /**
     * \brief Returns selection count
     */
    int selectionCount() const;

    /**
     * \brief Returns whether all visible contacts are selected
     */
    bool allContactsSelected() const;

    /**
     * \brief Set the list selection (multi-selection mode only)
     */
    void setSelection(const QList<QContactId> &selection);

    /**
     * \brief Returns current selection (multi-selection mode only)
     */
    QList<QContactId> selection() const;

    /**
     * \brief Selects all visible contacts
     */
    void selectAll();

    /**
     * \brief Unselects all visible contacts
     */
    void clearAll();

    /**
     * \brief Inserts the given widget in the viewport
     */
    void insertItem(MWidget *widget, int position, int stretch, Qt::Alignment alignment = 0);

    /**
     * \brief Removed the widget from viewport
     */
    void removeItem(MWidget *widget);

    /**
     * \brief Sets whether search widget should always be visible (even without any contacts)
     */
    void setForceSearchVisible(bool force);

    void setForceSearchFocus(bool force) { mForceSearchFocus = force; }

    SearchFieldWidget* searchWidget();

Q_SIGNALS:
    void itemClicked(ContactListItemData *data);
    void itemLongTapped(ContactListItemData *data, const QPointF &position);
    void modelChanged();
    void selectionChanged();
    void searchVisible(bool isVisible);

public Q_SLOTS:
    void update();

private Q_SLOTS:
    void onPanningStarted();
    void onPanningStopped();
    void onSearchStringChanged(const QString&);
    void onKeyboardStateChanged();
    void orientationChangeEvent(MOrientationChangeEvent *event);
    void onContactsUpdated();
    void removeStartupSpinner();

    void onItemClicked(const QModelIndex& index);
    void onItemLongTapped(const QModelIndex& index, const QPointF &position);
    void onSelectionChanged(const QItemSelection &selected, const QItemSelection &unselected);
    void onContactsAdded(const QList<QContactId> &ids);

    void updateListLabelVisibility();
    void updateSearchWidgetVisibility();
    void updateSearchWidgetVisibility(const QString &searchText);
    void updateSelection();
    void onSearchedContactSaved(const QContactId &id);

protected:
    void keyPressEvent(QKeyEvent *event);

protected:
    MKeyboardStateTracker   *mKeyboardTracker;
    MGridLayoutPolicy       *mViewportPolicy;

    ContactsListModel       *mListModel;
    ContactsGroupingModel   *mModelGrouped;
    ContactListItemBuilder  *mCellCreator;
    SearchFieldWidget       *mSearchWidget;
    ContactSearchWidget     *mContactSearchWidget;
    MStylableWidget         *mTopSpacer;
    MProgressIndicator      *mProgressIndicator;
    MLabel                  *mNoContactsLabel;
    MList                   *mList;
    QList<QContactId>        mSelection;
    MList::SelectionMode     mSelectionMode;

    bool mEnabledFastFiltering;
    M::Orientation mOrientation;
    bool mOrientationLocked;
    QString mPostponedSearchString;
    QString mSearchText;


    bool mListVisible;
    bool mSearchWidgetVisible;
    bool mNoContactsLabelVisible;
    bool mSelecting;
    bool mForceSearchFocus;
    bool mForceSearchVisible;
    bool mShowSearch;
    QContactId mNewContactId;

    QString mEmptyStateText;

private:
    void setSearchText(const QString &text);
    void removeProgressIndicator();
    void init();
    QString noContactsInDatabaseStr() const;
    QString searchStringNotFoundStr() const;
};

#endif  // CONTACTLISTWIDGET_H
