/* * This file is part of contacts *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTMODEL_H
#define CONTACTMODEL_H

// Mobility
#include <QContactManager>
#include <QContact>
#include <QContactAbstractRequest>
#include <QContactRelationship>
#include <QContactOnlineAccount>

// Us
#include "workerthread.h"
#include "contactaccountpath.h"

QTM_USE_NAMESPACE

using namespace com::nokia::contacts;

class ContactModelPrivate;
class ContactImportProcessor;
class QctContactMergeRequest;
class QctUnmergeIMContactsRequest;

/*!
 * \class ContactModel
 * Constructs a single instance of QContactModel - used in all places in app, to get
 * contacts. Calls are synchronous but not blocking on access to storage:
 * returned contacts are partially fetched and updated, after fetch finishes though \sa contactsUpdated()
 */
class ContactModel: public QObject
{
    Q_OBJECT

public:
    static ContactModel * instance ();
    static void cleanupInstance();
    virtual ~ContactModel();

    /*! Fetch given \a detailDefs for a list of contact \a ids, fetching contacts
     * from a cache if possible.
     *
     * The returned list of contacts will be the same size as the requested list of
     * \a ids, however, some may be incomplete.
     *
     * If all of the requested \a detailDefs cannot be supplied from cache, the
     * contact id is added to \a fetchingContacts if it is set, so the caller may
     * know it has an incomplete contact.
     *
     * If contacts are incomplete, you may listen to contactsUpdated() to get
     * the rest of the details.
     */
    QList<QContact> contacts(const QList<QContactId> &ids,
                             const QStringList &detailDefs,
                             QList<QContactId> *fetchingContacts = 0) const;

    /*! Fetch given \a details for a given contact \a id, from a cache if
     * possible.
     *
     * If you are fetching multiple contacts, please look into using the 
     * more efficient contacts() method instead, where you may additionally easily
     * determine if not all of the details were fetched from cache.
     */
    QContact contact(const QContactId &id, const QStringList &details = QStringList()) const;

    //!\ get contact with id from cache. This method will not trigger refresh fetch request.
    QContact contactFromCache(const QContactId &id) const;

    /*!\
     * Empty list if nothing is in cache, after fetch finished, relationshipsAdded is emitted.
     * \a role in which contactId is in relationship.
     * \a relationshipType - \sa QContactRelationship::relationshipType()
     */
    QList<QContactRelationship> contactRelationships(const QContactId &id, QContactRelationship::Role role = QContactRelationship::First, const QString &relationshipType = QString()) const;

    //!\ Convenience wrappers for \sa contactRelationships
    QList<QContactId> groupMembers(const QContactId &groupId) const;

    /*!
     * Returns a list of contact IDs that are merged with a given master contact \a id.
     * If there are no contacts merged with this ID, an empty list is returned.
     *
     * \sa contactRelationships()
     */
    QList<QContactId> mergedContactIds(const QContactId &id) const;

    /*!
     * Returns a list of contacts that are merged with a given master contact \a id.
     * Missing details are fetched if necessary.
     */
    QList<QContact> mergedContacts(const QContactId &id, const QStringList &details = QStringList()) const;

    /*!
     * When the contact is aggregated this method returns contact id for contact
     * that should aggregate \a contact - present it as a part of it.
     * e.g. when \a contact is in social manager, returned master contact is from tracker
     *
     * \returns empty QContactId() if no merging exists for the contact
     */
    QContactId masterContactId(const QContactId &contactId) const;

    /*!
     * \fn isMergedContact
     * Is social contact merged to another contact (which is the master contact of this contact)?
     */
    bool isMergedSocialContact(const QContactId &contactId) const;

    /*!
     * \fn isIMContactMerged
     * Is an IM contact (gtalk, skype, etc) merged with another contact (local/IM/social contact)
     */
    bool isIMContactMerged(const QContact &contact);

    // TODO should it return all contacts (this means ContactsListModel needs to filter them)
    // Or only contacts with type Contact or Group that will be present in contacts list
    // (this means ContactModelPrivate needs to separately store other type of contacts)
    // Currently, we just return all types of contacts
    // TODO If we don't need avatar widget and contact card to show Other type of contacts
    // We don't need to cache them in ContactModel. That would be much easier.
    /*!
     * \fn allContacts
     * \brief Returns all contacts from cache and issue fetch request to details that were not fetched before to cache.
     */
    QList<QContact> allContacts(const QStringList &details = QStringList()) const;

    /*!
     * \fn allContacts
     * \brief allContacts overload for specified set of managers.
     * \a limitToManagers defines to init only defined managers, not all available. Used for adding managers later (simcard manager)
     */
    QList<QContact> allContacts(const QStringList &details, const QStringList &limitToManagers) const;

    void allContactsAsync(const QStringList &details = QStringList());

    void setAutoMerge(bool autoMerge);

    /*!
     * \brief Used to detect if UI is running in contacts application or in client processes
     */
    bool isAutoMerging();

    /*!
    * Returns true if model is fetching data.
    */
    bool isFetching();

    //! Offers access to model's worker thread
    static WorkerThread &workerThread();

    //! Adds request and gives ownership to the model
    void addRequest(ContactModelAbstractRequest *request) const;
    void addRequest(QContactAbstractRequest *request) const;

    //! Do not process signals - do not keep ContactModel up to date
    void setSilent(bool silent);

    /*! This takes a \a list of QContactLocalId, and pretends that they were removed from
     * storage. This helps to compensate for managers who are slow at notifying
     * us of data save (like tracker).
     */
    void pretendToRemoveContacts(const QList<QContactId> &removedIds);

    void pretendToMergeContacts(const QList<QContactRelationship> &trackerRels, const QList<QContactRelationship> &socialRels);

    void pretendToUnmergeContacts(const QContact &contact, const QList<QContactOnlineAccount> &accounts, const QList<QContactRelationship> &socialRels);

    // clear unmatched contacts related functions
    void startClearUnmatchedTimer();
    void stopClearUnmatchedTimer();

Q_SIGNALS:
    void contactsUpdated (const QList<QContact> &contacts, const QStringList &updatedDetails);
    /*!
     * If client called allContacts(), there is no need to handle onContactsAdded signal
     * model fetches all new contacts and emits contactsUpdated signal when data is ready
     */
    void contactsAdded (const QList<QContactId> &contactIds);
    void contactsRemoved (const QList<QContactId> &contactIds);
    void contactsMerged(const QList<QContactRelationship> &relationships);
    void contactsUnmerged(const QList<QContactRelationship> &relationships);
    void relationshipsAdded(const QList<QContactRelationship> &relationships);
    void relationshipsRemoved(const QList<QContactRelationship> &relationships);
    void contactsNotFound(const QString &managerUri, const QList<QContactLocalId> &ids);
    void allContactsReady();
    // contactImportProcessed signal is emitted after imported contacts are handled by contactimportprocesser
    void contactImportProcessed(const ContactImportProcessor &result);

    void longProcessingStarted();
    void longProcessingFinished();
    void fetchingFinished();

private Q_SLOTS:
    void onContactsAdded(const QList<QContactLocalId>& contactIds);
    void onContactsChanged(const QList<QContactLocalId>& contactIds);
    void onContactsRemoved(const QList<QContactLocalId>& contactIds);
    void requestFinished(QContactAbstractRequest* request);
    void requestFinished(ContactModelAbstractRequest* request);
    void onRelationshipsAdded(const QList<QContactLocalId>& contactIds);
    void onRelationshipsRemoved(const QList<QContactLocalId>& contactIds);
    void onRelationshipsAdded(const QList<QContactRelationship> &relationships);
    void onRelationshipsRemoved(const QList<QContactRelationship> &relationships);
    void onModelRequestFinished();
    void onRequestAdded(QContactAbstractRequest* request);
    void onImportOnlyMatchingContactsChanged(const QString &accountPath, ContactAccountPath::PathType type, bool onlyMatching);
    void onDeferredWakeupInit();
    void onClearUnmatchedTimeout();
    void slotPrestartRestored();
    void slotPrestartReleased();

private:
    //!\ Private to make instantiation impossible
    ContactModel();
    void initManagerByUri(const QString &managerUri) const;
    void fetchContacts(const QString &managerUri, const QStringList &detailDefinitions, const QContactFilter &filter = QContactFilter()) const;
    void fetchAllContactIds(const QString &managerUri) const;
    void verifyContactList(const QString &managerUri, const QList<QContactLocalId> ids);

    void contactFetchRequestFinished(QContactAbstractRequest* request);
    void processFetchedContacts(const QList<QContact> &contacts, const QString &managerUri, const QStringList &detailDefinitions, const QContactFilter &filter = QContactFilter());

    void contactLocalIdFetchRequestFinished(QContactAbstractRequest* request);
    void relationshipFetchRequestFinished(QContactAbstractRequest* request);
    void relationshipSaveRequestFinished(QContactAbstractRequest* request);
    void contactSaveRequestStarted(QContactAbstractRequest* request);
    void contactSaveRequestFinished(QContactAbstractRequest* request);
    void contactRemoveRequestFinished(QContactAbstractRequest* request);
    void contactMergeRequestFinished(QctContactMergeRequest *merge);
    void mergeContacts(const QList<QContactRelationship> &mergeRelationships);
    void fetchRelationships(const QContactId &contactId, const QString &relationshipType = QString()) const;
    void fetchTrackerDiff() const;
    void processContactsAdded(const QString &managerUri, const QList<QContactLocalId>& contactIds);
    void processContactsChanged(const QString &managerUri, const QList<QContactLocalId>& contactIds);

    // update mapping of master/merged contacts from cached relationships
    void updateMergedContactsMapping() const;

    // remove all isSameAs relationships associated with contact id (whether it's master contact or merged contact)
    void removeMergeRelationshipsByContacts(const QList<QContactId> &ids);

    void fetchContactsByAccount(const QString &managerUri, const QString &accountPath, const QStringList &detailDefinitions) const;
    void removeUnmatchedContactsByAccount(const QString &managerUri, const QString &accountPath);

    void disconnectFromManager(QContactManager *manager) const;

private:
    ContactModelPrivate * const d;
    static ContactModel *_instance;
    friend class ContactModelCache;
    friend class ContactsModelUtils;
};

#endif // CONTACTMODEL_H

