/* * This file is part of contacts/contactui*
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTPICKERPAGE_H
#define CONTACTPICKERPAGE_H

#include <QTime>
#include <QList>

#include <MApplicationPage>

#include <QContact>

#include "contactsgroupingmodel.h"
#include "contactslistsettings.h"

QTM_USE_NAMESPACE

class ContactPickerPagePrivate;
class ContactListItemData;

class Q_DECL_EXPORT ContactPickerPage : public MApplicationPage
{
    Q_OBJECT

public:
    /**
     * \brief Constructor.
     *
     * \param modelSorted -the proxy model for this page. Users DON'T need to set its
     * source model, since this class will set it. The ContactPickerPage object
     * takes the ownership of modelSorted, and is responsible for deleting it
     * \param singleSelection - whether the pick page supports single or multiple selection
     * \param ids - list of pre-selected contact ids
     * \param autoCloseOnSelection - whether the picker should close on selection (single selection only)
     */
    explicit ContactPickerPage(ContactsGroupingModel *modelSorted,
                               bool singleSelection = true,
                               QList<QContactId> ids = QList<QContactId>(),
                               bool autoCloseOnSelection = true);

    virtual ~ContactPickerPage();

    virtual void createContent();

    /*!
     * Provides access to the model instance the picker uses to display content.
     * This can be used to e.g. set filtering options.
     */
    ContactsGroupingModel *model() const;

    /**
     * \brief Set a string to filter the contact list
     *
     * \param filter - a string to filter the contact list
     */
    void setFilter(const QString& filter);

    /**
     * \brief Returns the current filter
     *
     * \returns - a string containing the current filter
     */
    QString filter();

    /**
     * \brief Changes the title text
     */
    void setTitleText(const QString& text);

    /**
     * \brief Gets the current title text
     */
    QString titleText() const;

    /**
     * \brief Changes the text on the done button
     */
    void setDoneButtonText(const QString& text);

    /**
     * \brief Gets the current done button text
     */
    QString doneButtonText() const;

    /**
     * \brief Enable "Select/Clear All" button
     */
    void setSelectClearAllEnabled(bool enabled);

    /**
     * \brief Gets the current state of "Select/Clear All" button
     */
    bool isSelectClearAllEnabled() const;

    /**
     * \brief Enable the Import progress widget
     */
    void enableImportWidget(bool enable);

    /**
     * \brief Set minimum count of selections needed to enable done button
     */
    void setMinimumSelectionCount(int count);

Q_SIGNALS:
    /**
     * Emit this signal either when a user has selected contact(s) or
     * cancelled the contact picker.
     *
     * 1. In single selection mode, once a contact item is clicked,
     * emit this signal with the selected contact;
     * 2. In multiple selection mode, once the Done button is clicked,
     * emit this signal with all selected contacts;
     * 3. In either mode, if the back button is clicked (so the user has cancelled the picker),
     * emit this signal with an empty contact list.
     */
    void contactsPicked(const QList<QContactId> &);

    /**
     * Emit this signal either when a user has selected contact or
     * cancelled the contact picker.
     *
     * 1. In single selection mode, once a contact item is clicked,
     * emit this signal with the selected contact;
     * 2. In multiple selection mode, this signal is not emitted;
     * 3. In either mode, if the back button is clicked (so the user has cancelled the picker),
     * emit this signal with an empty contact list.
     */
    void contactPicked(const QContact &);

private Q_SLOTS:
    void updateTitle();
    void updateSelectClearAllButton();

    void onListItemClicked(ContactListItemData *data);
    void onToolbarDoneActionTriggered();
    void onBackButtonClicked();
    void onSelectClearAllButtonClicked();

    void onAppeared();
    void onDisappeared();
    void onImportProgressWidgetVisible(bool visible);

protected:
    //!\reimp from QObject
    bool eventFilter(QObject *obj, QEvent *event);

private:
    /* selectionDone() is called after a user has selected contact(s) */
    void selectionDone();
    void createToolbar();

    // Preselected contacts in picker
    QList<QContactId> mPreSelectedIds;
    bool mAutoCloseOnSelection;

    ContactPickerPagePrivate * const d;
};

#endif
