/* * This file is part of libcontacts*
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTPICKERSHEET_H
#define CONTACTPICKERSHEET_H

#include <QTime>
#include <QList>

#include <MList>
#include <MSheet>

#include <QContact>

#include "contactsgroupingmodel.h"
#include "contactslistsettings.h"

QTM_USE_NAMESPACE

class ContactPickerSheetPrivate;
class ContactListItemData;

class Q_DECL_EXPORT ContactPickerSheet : public MSheet
{
    Q_OBJECT

public:
    /**
     * \brief Constructor.
     *
     * \param modelSorted -the proxy model for this page. Users DON'T need to set its
     * source model, since this class will set it. The ContactPickerSheet object
     * takes the ownership of modelSorted, and is responsible for deleting it
     * \param selectionMode - the selection mode of the picker (single or multi selection)
     * \param groupBy - grouping mode for the picker list (alphabetically or by availability)
     * \param ids - list of contact id that need to be selected when opening the picker
     * \param autoCloseOnSelection - whether the picker is automatically closed on selection (single selection mode only)
     */
    explicit ContactPickerSheet(ContactsGroupingModel *modelSorted,
                                MList::SelectionMode selectionMode,
                                QList<QContactId> ids = QList<QContactId>(),
                                bool autoCloseOnSelection = true);

    virtual ~ContactPickerSheet();

    void createContent();

    /*!
     * Provides access to the model instance the picker uses to display content.
     * This can be used to e.g. set filtering options.
     */
    ContactsGroupingModel *model() const;

    /**
     * \brief Set a string to filter the contact list
     *
     * \param filter - a string to filter the contact list
     */
    void setFilter(const QString& filter);

    /**
     * \brief Returns the current filter
     *
     * \returns - a string containing the current filter
     */
    QString filter();

    /**
     * \brief Changes the title text
     */
    void setTitleText(const QString& text);

    /**
     * \brief Gets the current title text
     */
    QString titleText() const;

    /**
     * \brief Set empty state text
     */
    void setEmptyStateText(const QString &emptyStateText);

    /**
     * \brief Changes the text on the done button
     */
    void setDoneButtonText(const QString& text);

    /**
     * \brief Gets the current done button text
     */
    QString doneButtonText() const;

    /**
     * \brief Enable "Select/Clear All" button
     */
    void setSelectClearAllEnabled(bool enabled);

    /**
     * \brief Gets the current state of "Select/Clear All" button
     */
    bool isSelectClearAllEnabled() const;

    /**
     * \brief Enable the Import progress widget
     */
    void enableImportWidget(bool enable);

    /**
     * \brief Set minimum count of selections needed to enable done button
     */
    void setMinimumSelectionCount(int count);

Q_SIGNALS:
    /**
     * Emit this signal either when a user has selected contact(s) or
     * cancelled the contact picker.
     *
     * 1. In single selection mode, once a contact item is clicked,
     * emit this signal with the selected contact;
     * 2. In multiple selection mode, once the Done button is clicked,
     * emit this signal with all selected contacts;
     * 3. In either mode, if the back button is clicked (so the user has cancelled the picker),
     * emit this signal with an empty contact list.
     */
    void contactsPicked(const QList<QContactId> &);

    /**
     * Emit this signal either when a user has selected contact or
     * cancelled the contact picker.
     *
     * 1. In single selection mode, once a contact item is clicked,
     * emit this signal with the selected contact;
     * 2. In multiple selection mode, this signal is (temporarilly) emitted once per each selected contact.
     * This is deprecated, use contactsPicked(const QList<QContact> &).
     * 3. In either mode, if the back button is clicked (so the user has cancelled the picker),
     * emit this signal with an empty contact list.
     */
    void contactPicked(const QContact &);

    /**
     * Emit this signal either when a user has selected contacts or
     * cancelled the contact picker.
     *
     * If the back button is clicked (so the user has cancelled the picker),
     * emit this signal with an empty contact list.
     */
    void contactsPicked(const QList<QContact> &);

    void backButtonClicked();

public Q_SLOTS:
    void startSpinner();
    void stopSpinner();

private Q_SLOTS:
    void updateSelectClearAllAction();
    void onListItemClicked(ContactListItemData *data);
    void onDoneButtonClicked();
    void onCancelButtonClicked();
    void onSelectClearAllActionTriggered();
    void onImportProgressWidgetVisible(bool visible);

protected:
    //!\reimp from QObject
    bool eventFilter(QObject *obj, QEvent *event);

private:
    void createHeader();
    void createToolbar();

    ContactPickerSheetPrivate * const d;
};

#endif
