/* * This file is part of contacts/contactui *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTSGROUPINGMODEL_H_
#define CONTACTSGROUPINGMODEL_H_

// Qt
#include <QAbstractItemModel>
#include <QObject>
#include <QTimer>
#include <QRegExp>
#include <QUuid>
#include <QStringList>

// Mobility
#include <QContactId>

// MTF
#include <MAbstractItemModel>
#include <MLocale>
#include <MCollator>

// Us
class ContactListItemData;
class ContactsListModel;
#include "contactslistsettings.h"
#include "contactspickersortfilter.h"

QTM_USE_NAMESPACE

/*! ContactsGroupingModel is what you see when you look at a contact list.
 * It is effectively a tree proxy model of sorts. It takes a source (list model)
 * of contacts, and presents them in a grouped fashion, grouped by e.g. favorite status
 * and/or initials.
 */
class ContactsGroupingModel : public MAbstractItemModel
{
  Q_OBJECT

public:
    /**
     * \brief Constructor.
     * \param parent Parent QObject of this class.
     */
    explicit ContactsGroupingModel(QObject* parent=0);
    virtual ~ContactsGroupingModel();

    /**
     * Sets the grouping order for items and does the update.
     * \param order Grouping style for the items.
     */
    void setGrouping(ContactsListSettings::GroupByType grouping);

    /*!
     * \overload
     * Yes, it's totally stupid we have the *SAME ENUM* in two seperate APIs,
     * but unfortunately, we can't deprecate ContactsPickerSort, as it is used
     * by public code.
     *
     * TODO: remove this crap, someday?
     */
    void setGrouping(ContactsPickerSort::SortOrder grouping);

    void setMergeSuggestionId(const QContactId &contact);

    /*! Return the grouping type of the model.
     *
     * \sa setGrouping()
     */
    ContactsListSettings::GroupByType grouping() const;

    /*! Translate picker filtering data into stacked filters, used by this model.
     * This is required, because ContactsPickerFilter is exposed in public API, so we
     * (regretfully) cannot remove it.
     *
     * Note that clearFilters() will remove picker filters set on this model, and you will
     * need to call it again.
     *
     * TODO: remove this crap, someday?
     */
    void translatePickerFilters(ContactsPickerFilter::Filter filters, const QString &data);


    /*! Sets the string filter to look for \a filter inside contacts.
     * Contacts not matching this filter will not be displayed.
     * Make sure to call applyFilters() after changing the filter string.
     *
     * This is seperated from stackFilter() and clearFilters() to avoid the need
     * to restack filters every time the search string changes.
     *
     * The filter string operates as a mandatory filter.
     */
    void setFilterString(const QString &filter);

    const QRegExp &filterString() const;

    /*! Returns a normalised regex version of the filter.
     * This is used for search hilighting, so e.g. 'å' is turned into '[åa]'.
     */
    const QRegExp &uiFilterString() const;

    enum Filter {
        /*! Whether or not this contact is a favorite (bool)
         */
        IsFavorite,

        /*! Whether or not this contact is a group (bool)
         */
        IsGroup,

        /*! Whether or not this contact is editable (bool)
         */
        IsEditable,

        /*! Whether or not this contact has a phone number (bool)
         */
        HasPhoneNumber,

        /*! If this contact has an account with this path (QString)
         */
        HasAccountPath,

        /*! If this contact comes from this manager (QString)
         */
        HasManager,

        /*! If this contact *doesn't* come from this manager (QString)
         */
        ExcludeManager,

        /*! If this contact has a QContactActionDescriptor with the given name (QString)
         */
        HasCapability,

        /*! Only a given list of contacts (QList<QContactId>)
         */
        OnlyThese,

        /*! Not a given list of contacts (QList<QContactId>)
         */
        NotThese,

        /*! Contacts with locally stored details (bool)
         */
        IsExportEligible,

        /*! Contact is the Voicemail contact (bool)
         */
        IsVoicemail,

        /*! Whether or not this contact is removable (bool)
         */
        IsRemovable
    };

    enum FilterType
    {
        /*! This filter must pass, otherwise the contact is not a match.
         */
        Mandatory,

        /*! At least one filter of this type must pass,
         * otherwise the contact is not a match.
         */
        AtLeastOne
    };

    struct ListFilter
    {
        ListFilter(const Filter &f, const FilterType &ft, const QVariant &p)
            : filter(f)
            , filterType(ft)
            , parameter(p)
        {

        }

        Filter filter;
        FilterType filterType;
        QVariant parameter;
    };

    /*! Stacks a filter of a given \a filterType with the associated \a parameter for this model.
     * This is used to exclude (or include) contacts matching given parameters,
     * for instance:
     * stackFilter(IsFavorite, true)
     * stackFilter(IsGroup, false);
     * stackFilter(HasCapability, "StartVOIPCall")
     * applyFilters()
     * will match regular contacts who are favorites, who can have VOIP calls.
     */
    void stackFilter(Filter filter, const QVariant &parameter, FilterType filterType = Mandatory);

    /*! Apply the given filter string and stacked filters.
     * This is a seperate operation from stackFilter() and setFilterString() because it is
     * expensive; as it rebuilds ContactsGroupingModel from the source model, meaning an O(n)
     * operation at best (where 'n' is number of contacts) and O(n^m) at worst, where 'm'
     * is the number of filters.
     */
    void applyFilters();

    /*! Remove all filters from the model. This will mean all contacts from the source model
     * will be imported into ContactGroupingModel.
     *
     * Note that this does not clear the filter string set with setFilterString().
     */
    void clearFilters();

    /*! Returns all applied filters (except the filter string) set on this model.
     *
     * \sa filterString()
     */
    const QList<ListFilter> &filters() const;

    /*! Returns first HasAccountPath filter set on this model if any.
     */
    const QString accountPathFilter() const;

    /*! Returns whether grouping model is updated when main list model is changed.
     *  Update should be disabled while picker is disappearing.
     *  Default value is true.
     */
    bool isUpdateEnabled() const;

    /*! Sets whether the model should be enabled whenever the main list model is changed.
     */
    void setUpdateEnabled(bool enabled);

    //Overides:
    virtual void setModel(ContactsListModel *model);

    int groupCount() const;
    int rowCountInGroup(int group) const;
    QString groupTitle(int group) const;
    QVariant itemData(int row, int group, int role) const;
    int totalContacts() const;

signals:
    // signal is used for testing
    void contactsRegrouped();

private Q_SLOTS:
   void onRowsInserted(const QModelIndex&,int,int);
   void onRowsRemoved(const QModelIndex &parent, int start, int end);
   void onDataChanged(const QModelIndex &, const QModelIndex &);
   void onLayoutChanged();
   void localeChanged();

private:
   /*! Checks if the given list \a data item should be filtered.
    * Returns true if the row should be added to the model; false otherwise.
    */
    bool processFilterFor(ContactListItemData *data);

    void regroupContacts();

    /*!
     * Source model - this model acts like proxy for another (with flat hierarchy) model
     */
    ContactsListModel *sourceModel() const {
        return _sourceModel;
    }

    void connectModel(QAbstractItemModel *model);
    void disconnectModel(QAbstractItemModel *model);

    /*!
     * Used to handle updates - get data from source model and update this model.
     */
    ContactListItemData *sourceModelData(const QModelIndex &sourceModelIndex) const;

    /*!
    * Calculating group title for \a data based on current mGrouping
    */
    QString itemGroupTitle(const ContactListItemData *data) const;

    ContactsListSettings::GroupByType mGrouping;
    bool mIgnoreUpdates;
    ContactsListModel *_sourceModel;

    class Group;
    class Member;
    class Sortable;

    /*! List of 'parent' nodes in our tree. these are the root nodes under which contacts are placed.
     * Each Group contains a QList<ContactsGroupingModel::Member> which is, simply,
     * a container for the persistent model index of that contact in the source model.
     */
    QList<Group *> mGroups;

    /*! Finds a given group for given contact \a data, inserting it into the correct
     * position as necessary.
     */
    Group *findOrInsertGroupFor(ContactListItemData *data);

    /*! Remove a group by a given \a row.
     * Also notifies any views about the change.
     */
    void removeGroup(int row);

    /*! Inserts a given \a group into a given \a row.
     * Also notifies any views about the change.
     *
     * \sa findOrInsertGroupFor()
     */
    void insertGroup(int row, Group *group);

    /*! Finds the index of a given \a group in mGroups.
     * Returns -1 if the group cannot be found.
     */
    int indexOf(Group *group);

    /*! the key is the contactlistitemdata hash, to find group/member information.
     * this is used for example to efficiently add/remove rows instead of rebuilding the whole model.
     */
    QHash<QUuid, QPair<Group *, Member *> > mRowToGroupedIndexLookup;

    QList<ListFilter> mFilters;
    QRegExp mFilterString;
    QRegExp mUiFilterString;

    bool mUpdateEnabled;
    bool mEmitSignals;

    MLocale mLocale;
    MCollator mCollator;
    QStringList mCharactersIndex;
    ContactListItemData *mMergeSuggestionData;
};


#endif /* CONTACTSGROUPINGMODEL_H_ */
