/* * This file is part of libcontactswidgets *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTSGUI_H_
#define CONTACTSGUI_H_

#include <QAbstractItemModel>
#include <qcontact.h>

#include "contactspickersortfilter.h"
#include "contactsguiactionresult.h"

QTM_USE_NAMESPACE

class ContactPickerController;

class Q_DECL_EXPORT ContactsGui
{
public:
    /*!
     * \brief Creates and displays the contact picker GUI, blocking until the user has finished.
     *
     * For instance,
     * \code
     * ContactsGui::openContactPicker(
     *     this, SLOT(pickerContactsPicked(QList<QContactId>)),
     *     true,
     *     (ContactsPickerFilter::Filter)filter,
     *     (ContactsPickerSort::SortOrder)sorting);
     * \endcode
     *
     * \param The instance of the QObject whose \a slotPicked slot should be called when the user has picked the contacts.
     * \param The slot, of instance \a slotPickedInstance, to be called when the user has picked the contacts.
     * \param singleSelection - Whether the UI will offer only a single selection, or allow selection of multiple contacts.
     * \param filter - filter the contact picker to include certain contacts. By default the picker will show all contacts.
     * \param sorting - sort the contacts in the picker. By default the picker will sort them by name.
     */
    static void openContactPicker(QObject* slotInstance, const char* pickedSlot,
        bool singleSelection = true,
        ContactsPickerFilter::Filter filter = ContactsPickerFilter::None,
        ContactsPickerSort::SortOrder sorting = ContactsPickerSort::SortByName,
        QList<QContactId> preselected = QList<QContactId>());

    /*!
     * \brief Create a Contact Picker object
     * 
     * \param filterData - complements the filter variable, for example to specify AccountPath for ContactsPickerFilter::Account
     */
    static ContactPickerController *contactPicker(
        bool singleSelection = true,
        ContactsPickerFilter::Filter filter = ContactsPickerFilter::None,
        ContactsPickerSort::SortOrder sorting = ContactsPickerSort::SortByName,
        QList<QContactId> preselected = QList<QContactId>(), QString filterData = QString());

    /*!
     * \brief Create a Page for contact Picker
     * \deprecated Use \a contactPickerSheet instead
     */
    Q_DECL_DEPRECATED static MApplicationPage *contactPickerPage(QObject* slotInstance, const char* pickedSlot,
        bool singleSelection = true,
        ContactsPickerFilter::Filter filter = ContactsPickerFilter::None,
        ContactsPickerSort::SortOrder sorting = ContactsPickerSort::SortByName,
        QList<QContactId> preselected = QList<QContactId>());

    /*!
     * \brief Create a Page for contact Picker.
     * \deprecated Use \a contactPickerSheetSingleSelection instead
     */
    Q_DECL_DEPRECATED static MApplicationPage *contactPickerPageSingleSelection(QObject* slotInstance, const char* pickedSlot,
        ContactsPickerFilter::Filter filter = ContactsPickerFilter::None,
        ContactsPickerSort::SortOrder sorting = ContactsPickerSort::SortByName);

    /*!
     * \brief Create a Sheet for contact Picker
     */
    static MSheet *contactPickerSheet(QObject* slotInstance, const char* pickedSlot,
        bool singleSelection = true,
        ContactsPickerFilter::Filter filter = ContactsPickerFilter::None,
        ContactsPickerSort::SortOrder sorting = ContactsPickerSort::SortByName,
        QList<QContactId> preselected = QList<QContactId>());

    /*!
     * \brief Create a Sheet for contact Picker.
     * \code
     * ContactsGui::contactPickerSheetSingleSelection(
     *     this, SLOT(pickerContactPicked(QContact)),
     *     (ContactsPickerFilter::Filter)filter,
     *     (ContactsPickerSort::SortOrder)sorting);
     * \endcode
     */
    static MSheet *contactPickerSheetSingleSelection(QObject* slotInstance, const char* pickedSlot,
        ContactsPickerFilter::Filter filter = ContactsPickerFilter::None,
        ContactsPickerSort::SortOrder sorting = ContactsPickerSort::SortByName);

    /*!
     * \brief Creates and displays a contact picker sheet, allowing the user to choose contacts to merge.
     * This list will not show the being-merged \a contact from the contact list, and will sort the contacts
     * by their relevance for the merge.
     * The \a pickedSlot will be called once picker goes back. The bool parameter is true if contacts are picked; false if picker is cancelled.
     * Subsequently, contacts merging will start internally in this function. Once merging is complete (or failed),
     * the \a mergedSlot will be called with a bool parameter that indicates whetehr merging is sucessful or not.
     *
     * For instance,
     * \code
     * ContactsGui::openContactMergePicker(this, SLOT(onContactsPicked(bool)), SLOT(onMergingDone(bool)), currentContact);
     * \endcode
     *
     * \param slotInstance - The instance of the QObject whose \a pickedSlot slot should be called when merging is complete.
     * \param pickedSlot - The slot, of instance \a slotInstance, to be called when picker goes back (either contacts picked or picker cancelled).
     * \param mergedSlot - The slot, of instance \a slotInstance, to be called when merging is complete or failed.
     * \param contact - the being-merged contact.
     */
    static void openContactMergePicker(QObject* slotInstance, const char* pickedSlot, const char* mergedSlot, const QContact &contact);

    // This will be deprecated
    static void openContactMergePicker(QObject* slotInstance, const char* pickedSlot, const QContact &contact);

    // TODO API for transient contacts and prefilterring UI


    /*!
     * \brief displays editor for the contact and stores it to tracker when leaving editor
     *
     * \param contact - empty QContact() or existing contact in tracker.
     * If the contact does not exist in tracker then it will be created when the user leaves the editor,
     * unless all fields are cleared in the editor.
     * Clearing fields in the editor removes an existing contact if the contact
     * is allowed to be removed (emergency and trusted contacts probably are not).
     * This method returns immediately. It does not block until the editor is closed by the user.
     */
    //static void editContact(QContact &contact);
    static ContactsGuiActionResult::SharedPointer editContact(QContact &contact);

    //static void editContact(const QContactId &contactId);
    static ContactsGuiActionResult::SharedPointer editContact(const QContactId &contactId);

    /*!
     * \brief contact card for the contact
     *
     * \param contactId - identifies contact stored (existing) in tracker.
     * \sa libQtContacts QContactManager::contact(const QContactLocalId& contactId)
     */
    static ContactsGuiActionResult::SharedPointer openContactCard(const QContactLocalId &contactId);

    /*!
     * \brief contact card for the contact
     *
     * \param localContactId - local qcontact id
     * \param managerName -manager name, e.g. "social", "tracker"
     */
    static ContactsGuiActionResult::SharedPointer openContactCard(const QContactLocalId &localContactId,
                                                                  const QString &managerName);

    /*!
     * \brief contact card for the contact
     *
     * \param contactUri - identifies contact stored (existing) in tracker.
     */
    static ContactsGuiActionResult::SharedPointer openContactCard(const QString &contactUri);

    /*!
     * \brief contact card for the contact
     *
     * \param contact - contact stored or not in tracker or other QContactManager.
     * This method is important for showing transient and online contacts
     * that are not stored to tracker.
     */
    static ContactsGuiActionResult::SharedPointer openContactCard(const QContact &contact);

    /*!
     * \brief contact card for the contact and its merged social contacts
     *
     * \param contact - master contact (typically a tracker contact).
     * \param mergedContacts - contacts (typically social contacts) that are merged to the master contact
     * This method is for opening contact card from contact list. We can avoid fetching merged social contacts,
     * thus improving card opening performance
     */
    static ContactsGuiActionResult::SharedPointer openContactCard(const QContact &contact, const QList<QContact> &mergedContacts);

    /*!
     * \brief show contact list page
     */
    static ContactsGuiActionResult::SharedPointer openContactList();

    /*!
     * \brief show My Profile page
     */
    static void openMyProfile();

    /*!
     * \brief Launch send contact's vcard dialog.
     *
     * \a contact could be used to represent both persisted addressbook and transient contact
     */
    static void openSendContactCardDialog(const QContact &contact);

private:
    /*
     * \brief Helper method to open contact card
     */
    static ContactsGuiActionResult::SharedPointer showContactCard(QContactLocalId contactId, const QString &manager, bool external = false, MApplicationWindow *targetWindow = 0);
    /*
     * \brief Internal method which should be used inside contacts to open contact card
     */
    static ContactsGuiActionResult::SharedPointer openContactCardPrivate(const QContact &contact, const QList<QContact> &mergedContacts);
    static void restoreContactListPage(MApplicationWindow *targetWnd = 0);
    /*!
     * \brief Returns local id of contact currently opened in a contact card with deep link icon
     */
    static const QContactLocalId & deepLinkContactLocalId();

    /*!
     * \brief Setter for deep link contact local id
     */
    static void setDeepLinkContactLocalId(const QContactLocalId &id);

    static void openContactViaService(const QContactLocalId &id);

    /*!
     * \brief contact card for the contact
     *
     * \param window - identifies window in which contact card should be opened
     * \param contactId - identifies contact stored (existing) in tracker.
     * \sa libQtContacts QContactManager::contact(const QContactLocalId& contactId)
     */
    static ContactsGuiActionResult::SharedPointer openContactCard(MApplicationWindow *wnd, const QContactLocalId &contactId);

    /*!
     * \brief show filtered contact list page
     */
    static ContactsGuiActionResult::SharedPointer openFilteredContactList(const QString &accountPath, const QString &accountName);

    /*!
     * \brief Precreate card instance so that card opening is fast
     */
    static void precreateContactCard();

private:
    static QContactLocalId mDeepLinkLocalId;

friend class ContactListPage;
friend class CardContentWidget;
friend class FilterAction;
friend class ContactCardPage;
friend class ContactsService;
friend class AvatarSetter;
};

#endif /* CONTACTSGUI_H_ */
