/* * This file is part of contacts/contactui *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTSLISTMODEL_H_
#define CONTACTSLISTMODEL_H_

// Qt includes
#include <QObject>
#include <QAbstractListModel>
#include <QString>
#include <QQueue>
#include <QList>
#include <QSet>

// Qt Mobility Contacts includes
#include <qcontact.h>
#include <qcontactfetchrequest.h>

// People & Contacts includes
#include "icontactlistholder.h"
#include <contactmodel.h>

class ContactImportProcessor;
class ContactListItemData;

/**
 * \class ContactsListModel
 * This class stays between MLists and ContactModel - implements model in MVC.
 * Usage: MList::setModel(instanceOfThisClass)
 *
 * Not really public, marked public for autotests.
 */
class Q_DECL_EXPORT ContactsListModel: public QAbstractListModel, public IContactListHolder
{
Q_OBJECT

public:
    /**
     * \brief Construct list model with all know connection managers.
     * Not really public. See ::instance().
     * \param parent QObject parent of this class.
     */
    explicit ContactsListModel(QObject *parent = 0);

    virtual ~ContactsListModel();

    static ContactsListModel *instance();

    /**
     * \brief Returns the number of rows in the contact list.
     * \return The number of rows in the contact list.
     */
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    /**
     * \brief Returns the row value at an index as an QVariant.
     * \return The row value at an index as an QVariant.
     */
    virtual QVariant data(const QModelIndex &index, int role) const;

    /*!
     * Returns the list of cached items (ContactListItemData)
     */
    QList<ContactListItemData*> items();

    //!\reimp
    QList<QContact> getContactList();

    // special support for panning list use case - when panning, avoid processing updates from qtcontacts (that would change list model)
    void enableUpdatesProcessing();
    void disableUpdatesProcessing();

    /*!
     * Returns true if model is fetching data.
     */
    bool isFetching();

    /*!
     * Returns true if contacts exists in model (excluding voicemail, SIM, me and given contact).
     */
    bool hasContacts(const QContactId &excludedId = QContactId()) const;

    /*!
     * Returns true if contacts exists in model (excluding voicemail, SIM, me and given contact).
     * Sets hasVoicemail variable to true if there is a voicemail contact found
     */
    bool hasContacts(bool &hasVoicemail, const QContactId &excludedId = QContactId()) const;

    /*!
     * Returns true if more than one contact exists in model (excluding voicemail, SIM, me and given contact).
     * Sets hasVoicemail variable to true if there is a voicemail contact found
     */
    bool hasContactsMoreThanOne(bool &hasVoicemail, const QContactId &excludedId = QContactId()) const;

    /*! FIXME documentation */
    void clearAutoMergedContacts();

    /*! Add manager - add new one when needed, used for simcard manager (contacts displayed when Show is selected) */
    void addContactManager(const QString &managerName);

    /*! Retrieve the item data for a given contact id.
     */
    ContactListItemData *dataFor(const QContactId &id);

signals:
    /*!
     * Signal is emited when the first contact is added to the list (and the list is not empty)
     */
    void contactListNotEmpty();

    /*!
     * Signal is emited when all the contacts were successfully read.
     */
    void contactsUpdated();

    /*!
     * Emitted when new contacts are added.
     */
    void contactsAdded(const QList<QContactId> &);

    /*!
     * Emitted when contact fetching is finished
     */
    void fetchingFinished();

private slots:
    void onContactsUpdated(const QList<QContact> &contacts, const QStringList &updatedDetails);
    void onContactsRemoved(const QList<QContactId> &contactIds);
    void onContactsMerged(const QList<QContactRelationship> &relationships);
    void onContactsUnmerged(const QList<QContactRelationship> &relationships);
    void onNameOrderSettingChanged ();
    void onContactModelAllContactsReady ();
    void onContactImportProcessed(const ContactImportProcessor &result);

protected:
    void contactMerged(const QContactId &masterContactId, const QContactId &mergedContactId);
    void contactUnmerged(const QContactId &masterContactId, const QContactId &unmergedContactId);

    QList<QContact> mContacts;

    QHash<QContactId, int> mContactIdToRow;

    QHash<QContactId, ContactListItemData*> mCachedItemData;
    // mContactIds - when data is ready, speeding up the analysis about which contacts are new and which already existed (and need to be updated)
    QHash<QString, QSet<QContactLocalId> > mContactIds;

private slots:
    void startLongProcessing();
    void finishLongProcessing();

private:
    void initManagers();
    int rowIndexOf(const QContactId &id);
    void removeContacts(const QList<QContactId>& contactIds);
    void fillContactListItemData(ContactListItemData *itemData, const QContact &contact);

    void addUpdatedContacts(const QList<QContact> &contacts);
    void processUpdatedContacts();
    void processMergedContact(const QContact &contact);
    void processExistingContact(const QContact &contact);
    void processNewContact(const QContact &contact);
    QStringList detailDefinitionsUsedForList() const;

    QHash<QContactId, QContact> readyButNotProcessedContacts;

    bool mEnabledUpdatesProcessing;
    QHash< QString, QList<QContactAbstractRequest*> > managerPendingRequests;

    ContactModel *mContactModel;

    bool mContactsExists;

    int mLongProcessingCounter;

    friend class ContactListPage;
};


#endif /* CONTACTSLISTMODEL_H_ */
