/* * This file is part of libcontactswidgets *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef CONTACTSPAGEMANAGER_H_
#define CONTACTSPAGEMANAGER_H_

#include "contactlistpage.h"
#include "contactspickersortfilter.h"

class MApplicationPage;
class MSheet;
class ContactListPage;
class ContactPickerPage;
class ContactPickerSheet;

class Q_DECL_EXPORT ContactsPageManager
{
public:
    /*!
     * \brief Creates and displays the contact picker GUI, blocking until the user has finished.
     * This makes sure that the currently-active page will be reshown when the new page closes.
     * The page will be deleted when the back button is clicked, so the caller should not delete it.
     *
     * \param singleSelection - Whether the UI will offer only a single selection, or allow selection of multiple contacts.
     * \param filter - filter the contact picker to include certain contacts. By default the picker will show all contacts.
     * \param sorting - sort the contacts in the picker. By default the picker will sort them by name.
     * \param preselected - a list of contact id's that should be automatically pre-selected.
     * \param display - controls if the the picker page should open right away.
     * \param external - controls if picker is opened with transition or not
     * \param filterData - extra data to help the filtering. Example: AccountPath when filter is ContactsPickerFilter::Account.
     * \returns the page, so the caller can connect to its signals.
     */
    static ContactPickerPage* openContactPickerPage(bool singleSelection = true,
        ContactsPickerFilter::Filter filter = ContactsPickerFilter::None,
        ContactsPickerSort::SortOrder sorting = ContactsPickerSort::SortByName,
        QList<QContactId> preselected = QList<QContactId>(),
        bool display = true, bool external = false, QString filterData = QString());

    static ContactPickerSheet* openContactPickerSheet(bool singleSelection = true,
        ContactsPickerFilter::Filter filter = ContactsPickerFilter::None,
        ContactsPickerSort::SortOrder sorting = ContactsPickerSort::SortByName,
        QList<QContactId> preselected = QList<QContactId>(),
        bool display = true, bool external = false, QString filterData = QString());

    /*!
     * \brief Creates and displays a contact picker GUI for merging contact.
     * Except hiding the being-merged contact from the contact list, and sorting contacts by merging relevance,
     * the returned page behaves the same as ContactPickerSheet object created by openContactPicker().
     * The page will be deleted when the back button is clicked, so the caller should not delete it.
     *
     * \param contact - the being-merged contact
     * \param singleSelection - whether the UI will offer only a single selection, or allow selection of multiple contacts
     * \returns the page, so the caller can connect to its signals.
     */
    static ContactPickerSheet* openContactMergePickerSheet(const QContact &contact, bool singleSelection = true);

    /*!
     * Displays a \a page, regardless of whether the page is
     * in an existing application with a window or in a windowless environment.
     *
     * Also raises the application window to the top.
     *
     * The page lifecycle varies, depending on the \a deletionPolicy provided.
     */
    static void displayPage(MApplicationPage *page, MSceneWindow::DeletionPolicy deletionPolicy, bool external = false);

    /*!
     * Displays a \a page, in a particular \a targetWindow regardless of whether the page is
     * in an existing application with a window or in a windowless environment.
     * If no \a window is found among existing  application's windows does the same as displayPage()
     *
     * Also raises the application window to the top.
     *
     * The page lifecycle varies, depending on the \a deletionPolicy provided.
     */
    static void displayPageInWindow(MApplicationWindow *targetWindow, MApplicationPage *page, MSceneWindow::DeletionPolicy deletionPolicy, bool external = false);

    /*!
     * Displays a \a sheet
     */
    static void displaySheet(MSheet *sheet, MSceneWindow::DeletionPolicy deletionPolicy, bool external = false);

private:
    static void displaySceneWindow(MSceneWindow *sceneWnd, MSceneWindow::DeletionPolicy deletionPolicy, bool external = false, MApplicationWindow *targetWnd = 0);
};

#endif /* CONTACTSPAGEMANAGER_H_ */
