#ifndef SIMDATASTORE_H
#define SIMDATASTORE_H

// Qt
#include <QObject>
#include <QList>
#include <QString>

//
//  W A R N I N G
//  -------------
//
// This file is not part of the API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

class SimDataStorePrivate;

// cellular-qt
#include <SIM>
#include <SIMPhonebook>
using namespace Cellular;
using namespace Cellular::SIM;

/**
 * Singleton class that provides access to various data found on the SIM card and
 * data about the SIM card (used storage, field lengths).
 *
 * This class is more or less a hack to provide access to data that previously was
 * already available inside QtContactSIMBackendCellularQt but not accessible (due to
 * the plugin-nature of qtcontacts-simcard). Therefore must functionality is only
 * available QtContactSIMBackendCellularQt.
 */
class SimDataStore : public QObject
{
    friend class QtContactSIMBackendCellularQt;
    friend class SimDataStorePrivate;

    Q_OBJECT
public:
    static const int VMC_INDEX;

    static SimDataStore* instance();

    /// return true when a card is present and unlocked
    bool cardPresent();

    /// returns true if the currently plugged SIM has a voicemail number
    bool voicemailAvailable() const;

    /// returns true if we are currently reading from the SIM
    bool isFetchOngoing();

    /// describes the max fields lengths supported by the currently plugged SIM
    struct FieldLengthInfo {
        FieldLengthInfo();

        uint name;
        uint number;
        uint sne;
        uint snr;
        uint email;
    };
    /// returns the field length restrictions for the currently plugged SIM
    FieldLengthInfo fieldLengthInfo();

    /// describes the storage space available on the currently plugged SIM
    /// for each fields on the card there is a "max" and a "used" property.
    /// "max" describes the maximum limit while "used" describes the number of
    /// actually used fields on the SIM.
    struct StorageInfo {
        StorageInfo();

        uint maxEntries;
        uint usedEntries;
        uint maxSneEntries;
        uint usedSneEntries;
        uint maxSnrEntries;
        uint usedSnrEntries;
        uint maxEmailEntries;
        uint usedEmailEntries;
    };
    /// returns the storage info for the currently plugged SIM
    StorageInfo storageInfo();

protected:
    struct Contact {
        Contact();
        Contact(int index, QString name, QString number, QString sne,
                QString snr, QString email);

        int index;
        QString name;
        QString number;
        QString sne;
        QString snr;
        QString email;
    };

    /// request to fetch all the contacts from the SIM. the result
    /// is available through the fetchComplete(QList<SimDataStore::Contact>) signal.
    void fetch();

    void deleteEntry(SIMPhonebook::PhonebookType type, int index);
    void createEntry(SIMPhonebook::PhonebookType type, SimDataStore::Contact contact);
    void updateEntry(SIMPhonebook::PhonebookType type, int index, SimDataStore::Contact contact);

signals:
    void fetchComplete(QList<SimDataStore::Contact>);
    void deleteEntryComplete();
    void createEntryComplete();
    void updateEntryComplete();
    void cardPresent(bool);
    void voicemailPresent(bool);

private:
    SimDataStore();
    ~SimDataStore();

    SimDataStorePrivate* d;
    static SimDataStore* mInstance;
};

#endif // SIMDATASTORE_H
