/* * This file is part of contactui *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef VOICEMAIL_H
#define VOICEMAIL_H

#include <QtCore/QDebug>
#include <QtCore/QObject>
#include <QtCore/QFlags>
#include <QtCore/QString>
#include <QtCore/QList>

// forward declarations
class VoicemailNumberPrivate;
class VoicemailPrivate;

class VoicemailNumber
{
public:
    enum Type {
        Normal    = 0x0,
        Preferred = 0x1,
        FromSIM   = 0x2
    };

    Q_DECLARE_FLAGS(Types, Type)

public:
    VoicemailNumber();
    VoicemailNumber(const QString &number, VoicemailNumber::Types types);
    explicit VoicemailNumber(const QString &number);
    VoicemailNumber(const VoicemailNumber &number);
    virtual ~VoicemailNumber();

    VoicemailNumber &operator=(const VoicemailNumber &number);
    bool operator==(const VoicemailNumber &number) const;

    bool operator<(const VoicemailNumber&) const;

    //! Set voicemail number
    void setNumber(const QString &number);
    //! Set voicemail number as preferred number
    void setPreferred(bool preferred);
    //! Set voicemail number as number from SIM
    void setFromSIM(bool fromSIM);

    QString number() const;
    bool isPreferred() const;
    bool isFromSIM() const;
private:
    VoicemailNumberPrivate * const d;
};

class Voicemail: public QObject
{
    Q_OBJECT

public:

    //! Possible Voicemail errors
    enum VoicemailError
    {
        NoError,           /// no error happened, operation completed successfully
        Unknown,           /// general Voicemail failure
        NoPreferred,       /// no preferred number found
        MultiplePreferred, /// found multiple preferred numbers while saving to store. Note: saving was completed, but only first marked number is saved as preferred
        SIMError,          /// could not get voicemail number from SIM (SIMPhonebook failure)
    };

public:
    Voicemail(QObject *parent=0);

    virtual ~Voicemail();

public Q_SLOTS: // METHODS
    //! Request preferred voicemail number
    virtual void queryPreferredVoicemailNumber();

    //! Request all voicemail numbers
    virtual void queryVoicemailNumbers();

Q_SIGNALS: // SIGNALS
    //! Emitted when preferredVoicemailNumber() call completes
    void preferredVoicemailNumber(const QString &preferredVoicemailNumber,
                                  Voicemail::VoicemailError error);

    //! Emitted when preferred VoicemailNumber changes
    void preferredVoicemailNumberChanged(const QString &preferredVoicemailNumber,
                                         Voicemail::VoicemailError error);

    //! Emitted when voicemailNumbers() call completes
    void voicemailNumbers(const QList<VoicemailNumber> &voicemailNumbers,
                          Voicemail::VoicemailError error);

    //! Emitted when list of VoicemailNumbers changes
    void voicemailNumbersChanged(const QList<VoicemailNumber> &voicemailNumbers,
            Voicemail::VoicemailError error);

private:
    VoicemailPrivate * const d;

private Q_SLOTS:
    void onStoreInitComplete();
};

Q_DECLARE_OPERATORS_FOR_FLAGS(VoicemailNumber::Types)
QDebug operator<<(QDebug dbg, const VoicemailNumber &number);

#endif // VOICEMAIL_H
