/* * This file is part of contactui *
 * Copyright © 2009 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
 * Contact: Aleksandar Stojiljkovic <aleksandar.stojiljkovic@nokia.com>
 * This software, including documentation, is protected by copyright controlled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others without the prior written consent of Nokia. */

#ifndef VOICEMAILSTORE_H
#define VOICEMAILSTORE_H

// Qt includes
#include <QtCore/QDebug>
#include <QtCore/QObject>
#include <QtCore/QFlags>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QFileSystemWatcher>
#include <QtCore/QQueue>

#include <qtcontacts.h>

QTM_USE_NAMESPACE

// Cellular Qt includes
#include <SIMPhonebook>

// People & Contacts includes
#include "voicemail.h"

class VoicemailStorePrivate;

using namespace Cellular;

/** This is NOT a public API - code could change any time.
    Do not rely on it outside if the people codebase, use the
    public voicemail.h wrapper instead!
  */
class VoicemailStore: public QObject
{
    Q_OBJECT

public:
    /** Note: the simPhonebook argument is not used, its just there to keep
              the ABI from breaking. */
    explicit VoicemailStore(SIMPhonebook *simPhonebook =0, QObject *parent=0);

    ~VoicemailStore();

    static const QLatin1String voicemailContactGUID;

public Q_SLOTS: // METHODS
    //! Request preferred voicemail number. The result is delivered through the
    //! preferredVoicemailNumber() signal.
    void queryPreferredVoicemailNumber();

    //! Request all voicemail numbers. The result is delivered through the
    //! voicemailNumbers() signal.
    void queryVoicemailNumbers();

    /** Request to store voicemail numbers
        Used by external clients (for example the editordialog) to store voicemail numbers.
        This methods overides all phone details currently attached to the VMC with
        'voicemailNumbers'. No merging is done.
        Example: For the case that the user swaps the SIM while in the editor the editor might
        override the number just imported from the new SIM. I don't consider this serious as
        a) its a rare/specific situation, b) its easy to overcome by the user
        (just pop-in the SIM again).
    */
    void storeVoicemailNumbers(const QList<VoicemailNumber> &voicemailNumbers);

    //! request the store to update itself from a SIM card or the pre-customization.
    //! if there are relevant changes the result will be stored to tracker.
    void selfUpdate();

    //! Set details for voicemail contact
    static void setVoicemailCommonDetails(QContact &contact);

Q_SIGNALS: // SIGNALS
    //! Emitted when queryPreferredVoicemailNumber() call completes
    void preferredVoicemailNumber(const QString &preferredVoicemailNumber,
                                  Voicemail::VoicemailError error);

    //! Emitted when preferred VoicemailNumber changes
    void preferredVoicemailNumberChanged(const QString &preferredVoicemailNumber,
                                         Voicemail::VoicemailError error);

    //! Emitted when queryVoicemailNumbers() call completes
    void voicemailNumbers(const QList<VoicemailNumber> &voicemailNumbers,
                          Voicemail::VoicemailError error);

    //! Emitted when list of VoicemailNumbers changes
    void voicemailNumbersChanged(const QList<VoicemailNumber> &voicemailNumbers,
                                 Voicemail::VoicemailError error);

    //! Emitted when storeVoicemailNumbers() call completes
    void storeVoicemailNumbersComplete(Voicemail::VoicemailError error);

    /** Emitted when the store has completed its initalization. Its not allowed
        to call any public methods of the store before it is initialzed. */
    void initComplete();

    //! Emitted when the selfUpdate() calls completes.
    void selfUpdateComplete();

private:
    VoicemailStorePrivate* d;
};

#endif // VOICEMAIL_H
